@@ -1,40 +1,39 @@
 from collections import defaultdict
-from typing import Any, Dict, List, Tuple, cast
+from typing import Any, List
 
 from docutils import nodes
-from sphinx.util.docutils import SphinxDirective
-from sphinx.directives import ObjectDescription
-from docutils.parsers.rst import directives
 from docutils.nodes import Node
-from sphinx.domains import Domain
-from sphinx.domains import Index
-from sphinx.roles import XRefRole
-from sphinx.util.nodes import make_refnode
-from sphinx.util.docfields import DocFieldTransformer
+from docutils.parsers.rst import directives
+from rdflib import Graph, Literal, Namespace  # type: ignore
+from rdflib.namespace import RDF, RDFS  # type: ignore
 from sphinx import addnodes
-
-
-from rdflib import Graph, Namespace, Literal
-from rdflib.namespace import RDF, RDFS
+from sphinx.directives import ObjectDescription
+from sphinx.domains import Domain, Index
+from sphinx.roles import XRefRole
+from sphinx.util.docfields import DocFieldTransformer
+from sphinx.util.docutils import SphinxDirective
+from sphinx.util.nodes import make_refnode
 
 
 class TTL(SphinxDirective):
     has_content = True
 
     def run(self):
-        if not hasattr(self.env, 'probs_all_ttl'):
+        if not hasattr(self.env, "probs_all_ttl"):
             self.env.probs_all_ttl = []
 
-        self.env.probs_all_ttl.append({
-            'docname': self.env.docname,
-            'lineno': self.lineno,
-            'content': self.content,
-            # 'target': targetnode,
-        })
+        self.env.probs_all_ttl.append(
+            {
+                "docname": self.env.docname,
+                "lineno": self.lineno,
+                "content": self.content,
+                # 'target': targetnode,
+            }
+        )
         # print(self.env.docname, self.lineno)
         # print(self.content)
 
-        paragraph_node = nodes.literal_block(text='\n'.join(self.content))
+        paragraph_node = nodes.literal_block(text="\n".join(self.content))
         return [paragraph_node]
 
 
@@ -47,7 +46,7 @@ class StartSubProcessesDirective(SphinxDirective):
         return Namespace(self.config.probs_rdf_system_prefix)
 
     def run(self):
-        if not hasattr(self.env, 'probs_parent'):
+        if not hasattr(self.env, "probs_parent"):
             self.env.probs_parent = []
 
         uri = getattr(self.SYS, self.arguments[0])
@@ -66,7 +65,7 @@ class StartSubObjectsDirective(SphinxDirective):
         return Namespace(self.config.probs_rdf_system_prefix)
 
     def run(self):
-        if not hasattr(self.env, 'probs_parent_object'):
+        if not hasattr(self.env, "probs_parent_object"):
             self.env.probs_parent_object = []
 
         uri = getattr(self.SYS, self.arguments[0])
@@ -78,25 +77,29 @@ class StartSubObjectsDirective(SphinxDirective):
 
 class EndSubProcessesDirective(SphinxDirective):
     def run(self):
-        if not hasattr(self.env, 'probs_parent'):
+        if not hasattr(self.env, "probs_parent"):
             self.env.probs_parent = []
         if self.env.probs_parent:
             leaving = self.env.probs_parent.pop()
-            paragraph_node = nodes.literal_block(text=f'Ending sub processes of "{leaving}"')
+            paragraph_node = nodes.literal_block(
+                text=f'Ending sub processes of "{leaving}"'
+            )
         else:
-            paragraph_node = nodes.literal_block(text=f'Nothing to end!')
+            paragraph_node = nodes.literal_block(text="Nothing to end!")
         return [paragraph_node]
 
 
 class EndSubObjectsDirective(SphinxDirective):
     def run(self):
-        if not hasattr(self.env, 'probs_parent_object'):
+        if not hasattr(self.env, "probs_parent_object"):
             self.env.probs_parent_object = []
         if self.env.probs_parent_object:
             leaving = self.env.probs_parent_object.pop()
-            paragraph_node = nodes.literal_block(text=f'Ending sub objects of "{leaving}"')
+            paragraph_node = nodes.literal_block(
+                text=f'Ending sub objects of "{leaving}"'
+            )
         else:
-            paragraph_node = nodes.literal_block(text=f'Nothing to end!')
+            paragraph_node = nodes.literal_block(text="Nothing to end!")
         return [paragraph_node]
 
 
@@ -104,8 +107,8 @@ PROBS = Namespace("https://ukfires.org/probs/ontology/")
 
 
 def remove_ns(ns, uri):
-    if uri[:len(ns)] == ns:
-        return uri[len(ns):]
+    if uri[: len(ns)] == ns:
+        return uri[len(ns) :]
 
 
 class SystemObjectDescription(ObjectDescription):
@@ -128,8 +131,8 @@ class SystemObjectDescription(ObjectDescription):
         Main directive entry function, called by docutils upon encountering the
         directive.
 
-        This directive is meant to be quite easily subclassable, so it delegates
-        to several additional methods.  What it does:
+        This directive is meant to be quite easily subclassable, so it
+        delegates to several additional methods.  What it does:
 
         * find out if called as a domain-specific directive, set self.domain
         * create a `desc` node to fit all description inside
@@ -140,30 +143,31 @@ class SystemObjectDescription(ObjectDescription):
           or raise ValueError
         * add index entries using self.add_target_and_index()
         * parse the content and handle doc fields in it
+
         """
-        if ':' in self.name:
-            self.domain, self.objtype = self.name.split(':', 1)
+        if ":" in self.name:
+            self.domain, self.objtype = self.name.split(":", 1)
         else:
-            self.domain, self.objtype = '', self.name
+            self.domain, self.objtype = "", self.name
         self.indexnode = addnodes.index(entries=[])
 
         nest_depth = "nested-%d" % self.get_nesting_depth()
         node = nodes.admonition(classes=["toggle", self.objtype, nest_depth])
         node.document = self.state.document
-        node['domain'] = self.domain
+        node["domain"] = self.domain
         # 'desctype' is a backwards compatible attribute
-        node['objtype'] = node['desctype'] = self.objtype
-        node['noindex'] = noindex = ('noindex' in self.options)
+        node["objtype"] = node["desctype"] = self.objtype
+        node["noindex"] = noindex = "noindex" in self.options
         if self.domain:
-            node['classes'].append(self.domain)
+            node["classes"].append(self.domain)
 
         self.names = []  # type: List[Any]
         signatures = self.get_signatures()
-        assert len(signatures) == 1, 'only assuming 1 signature can be given'
+        assert len(signatures) == 1, "only assuming 1 signature can be given"
         for i, sig in enumerate(signatures):
             # add a signature node for each signature in the current unit
             # and add a reference target for it
-            signode = nodes.title(sig, '')
+            signode = nodes.title(sig, "")
             self.set_source_info(signode)
             node.append(signode)
             try:
@@ -180,21 +184,23 @@ class SystemObjectDescription(ObjectDescription):
                 self.names.append(name)
                 if not noindex:
                     # only add target and index entry if this is the first
-                    # description of the object with this name in this desc block
+                    # description of the object with this name in this desc
+                    # block
                     self.add_target_and_index(name, sig, signode)
 
         contentnode = nodes.paragraph()
         node.append(contentnode)
         if self.names:
             # needed for association of version{added,changed} directives
-            self.env.temp_data['object'] = self.names[0]
+            self.env.temp_data["object"] = self.names[0]
         self.before_content()
         self.state.nested_parse(self.content, self.content_offset, contentnode)
         self.transform_content(contentnode)
-        self.env.app.emit('object-description-transform',
-                          self.domain, self.objtype, contentnode)
+        self.env.app.emit(
+            "object-description-transform", self.domain, self.objtype, contentnode
+        )
         DocFieldTransformer(self).transform_all(contentnode)
-        self.env.temp_data['object'] = None
+        self.env.temp_data["object"] = None
         self.after_content()
 
         # Do this here so `become_parent` hasn't taken effect too early
@@ -206,7 +212,9 @@ class SystemObjectDescription(ObjectDescription):
         signode += nodes.emphasis(self.signature_prefix, self.signature_prefix)
         signode += addnodes.desc_name(sig, sig)
         if "label" in self.options:
-            signode += nodes.emphasis(self.options["label"], " / " + self.options["label"])
+            signode += nodes.emphasis(
+                self.options["label"], " / " + self.options["label"]
+            )
         return sig
 
 
@@ -214,11 +222,11 @@ class Process(SystemObjectDescription):
     # has_content = True
     # required_arguments = 1
     option_spec = {
-        'label': directives.unchanged_required,
-        'become_parent': directives.flag,
-        'consumes': directives.unchanged,
-        'produces': directives.unchanged,
-        'composed_of': directives.unchanged,
+        "label": directives.unchanged_required,
+        "become_parent": directives.flag,
+        "consumes": directives.unchanged,
+        "produces": directives.unchanged,
+        "composed_of": directives.unchanged,
     }
     signature_prefix = "Process: "
 
@@ -241,13 +249,17 @@ class Process(SystemObjectDescription):
     def add_target_and_index(self, name_cls, sig, signode):
         # print('add_target_and_index', name_cls, sig, signode)
         # print(self.options)
-        signode['ids'].append('process' + '-' + sig)
-        if 'noindex' not in self.options:
-            recipes = self.env.get_domain('system')
-            recipes.add_process(sig, self.options.get('consumes', '').split(), self.options.get('produces', '').split())
+        signode["ids"].append("process" + "-" + sig)
+        if "noindex" not in self.options:
+            recipes = self.env.get_domain("system")
+            recipes.add_process(
+                sig,
+                self.options.get("consumes", "").split(),
+                self.options.get("produces", "").split(),
+            )
 
     def define_graph(self, uri_str):
-        if not hasattr(self.env, 'probs_graph'):
+        if not hasattr(self.env, "probs_graph"):
             g = Graph()
             g.bind("sys", self.SYS)
             g.bind("probs", PROBS)
@@ -263,8 +275,10 @@ class Process(SystemObjectDescription):
         g.add((uri, RDF.type, PROBS.Process))
         if "label" in self.options:
             g.add((uri, RDFS.label, Literal(self.options["label"])))
+            g.add((uri, PROBS.processName, Literal(self.options["label"])))
         else:
             g.add((uri, RDFS.label, Literal(uri_str)))
+            g.add((uri, PROBS.processName, Literal(uri_str)))
 
         # ComposedOf relationships
         if self.env.probs_parent:
@@ -296,9 +310,9 @@ class Object(SystemObjectDescription):
     has_content = True
     required_arguments = 1
     option_spec = {
-        'label': directives.unchanged,
-        'become_parent': directives.flag,
-        'parent_object': directives.unchanged,
+        "label": directives.unchanged,
+        "become_parent": directives.flag,
+        "parent_object": directives.unchanged,
     }
     signature_prefix = "Object: "
 
@@ -306,7 +320,8 @@ class Object(SystemObjectDescription):
     #     signode += nodes.emphasis("Object: ", "Object: ")
     #     signode += addnodes.desc_name(sig, sig)
     #     if "label" in self.options:
-    #         signode += addnodes.desc_addname(self.options["label"], self.options["label"])
+    #         signode += addnodes.desc_addname(self.options["label"],
+    #                                          self.options["label"])
     #     return sig
 
     def transform_content(self, contentnode):
@@ -323,13 +338,13 @@ class Object(SystemObjectDescription):
     def add_target_and_index(self, name_cls, sig, signode):
         # print('add_target_and_index', name_cls, sig, signode)
         # print(self.options)
-        signode['ids'].append('object' + '-' + sig)
-        if 'noindex' not in self.options:
-            recipes = self.env.get_domain('system')
+        signode["ids"].append("object" + "-" + sig)
+        if "noindex" not in self.options:
+            recipes = self.env.get_domain("system")
             recipes.add_object(sig, [])
 
     def define_graph(self, uri_str):
-        if not hasattr(self.env, 'probs_graph'):
+        if not hasattr(self.env, "probs_graph"):
             g = Graph()
             g.bind("sys", self.SYS)
             g.bind("probs", PROBS)
@@ -345,8 +360,10 @@ class Object(SystemObjectDescription):
         g.add((uri, RDF.type, PROBS.Object))
         if "label" in self.options:
             g.add((uri, RDFS.label, Literal(self.options["label"])))
+            g.add((uri, PROBS.objectName, Literal(self.options["label"])))
         else:
             g.add((uri, RDFS.label, Literal(uri_str)))
+            g.add((uri, PROBS.objectName, Literal(uri_str)))
 
         if "parent_object" in self.options:
             parent_uri = getattr(self.SYS, self.options["parent_object"])
@@ -362,34 +379,38 @@ class Object(SystemObjectDescription):
 class ObjectIndex(Index):
     """Index of objects."""
 
-    name = 'object'
-    localname = 'Object Index'
-    shortname = 'Object'
+    name = "object"
+    localname = "Object Index"
+    shortname = "Object"
 
     def generate(self, docnames=None):
         content = defaultdict(list)
 
-        objects = {name: (dispname, typ, docname, anchor)
-                   for name, dispname, typ, docname, anchor, _
-                   in self.domain.data['objects']}
-        processes = {name: (dispname, typ, docname, anchor)
-                     for name, dispname, typ, docname, anchor, _
-                     in self.domain.data['processes']}
-        process_consumes = self.domain.data['process_consumes']
-        process_produces = self.domain.data['process_produces']
+        objects = {
+            name: (dispname, typ, docname, anchor)
+            for name, dispname, typ, docname, anchor, _ in self.domain.data["objects"]
+        }
+        processes = {
+            name: (dispname, typ, docname, anchor)
+            for name, dispname, typ, docname, anchor, _ in self.domain.data["processes"]
+        }
+        process_consumes = self.domain.data["process_consumes"]
+        process_produces = self.domain.data["process_produces"]
         object_processes = defaultdict(list)
 
         # flip from recipe_ingredients to ingredient_recipes
         for process_name, objs in process_consumes.items():
             for obj in objs:
-                if ('object.' + obj) not in objects:
-                    print('WARNING: Object {} consumed by process {} is not defined'.format(obj, process_name))
+                if ("object." + obj) not in objects:
+                    msg = "WARNING: Object {} consumed by process {} is not defined"
+                    print(msg.format(obj, process_name))
                     continue
                 object_processes[obj].append((process_name, "consumed"))
         for process_name, objs in process_produces.items():
             for obj in objs:
-                if ('object.' + obj) not in objects:
-                    print('WARNING: Object {} produced by process {} is not defined'.format(obj, process_name))
+                if ("object." + obj) not in objects:
+                    msg = "WARNING: Object {} produced by process {} is not defined"
+                    print(msg.format(obj, process_name))
                     continue
                 object_processes[obj].append((process_name, "produced"))
 
@@ -398,15 +419,13 @@ class ObjectIndex(Index):
         #
         # name, subtype, docname, anchor, extra, qualifier, description
         for obj, process_names in object_processes.items():
-            dispname, typ, docname, anchor = objects['object.' + obj]
+            dispname, typ, docname, anchor = objects["object." + obj]
             k = dispname[0].lower()
-            content[k].append(
-                (dispname, 1, docname, anchor, docname, '', typ))
+            content[k].append((dispname, 1, docname, anchor, docname, "", typ))
 
             for process_name, direction in process_names:
                 dispname, typ, docname, anchor = processes[process_name]
-                content[k].append(
-                    (dispname, 2, docname, anchor, direction, '', typ))
+                content[k].append((dispname, 2, docname, anchor, direction, "", typ))
 
         # convert the dict to the sorted list of tuples expected
         content = sorted(content.items())
@@ -417,15 +436,15 @@ class ObjectIndex(Index):
 class ProcessIndex(Index):
     """Index of processes."""
 
-    name = 'process'
-    localname = 'Process Index'
-    shortname = 'Process'
+    name = "process"
+    localname = "Process Index"
+    shortname = "Process"
 
     def generate(self, docnames=None):
         content = defaultdict(list)
 
         # sort the list of processes in alphabetical order
-        processes = self.domain.data['processes']
+        processes = self.domain.data["processes"]
         processes = sorted(processes, key=lambda process: process[0])
 
         # generate the expected output, shown below, from the above using the
@@ -434,7 +453,8 @@ class ProcessIndex(Index):
         # name, subtype, docname, anchor, extra, qualifier, description
         for name, dispname, typ, docname, anchor, _ in processes:
             content[dispname[0].lower()].append(
-                (dispname, 0, docname, anchor, docname, '', typ))
+                (dispname, 0, docname, anchor, docname, "", typ)
+            )
 
         # convert the dict to the sorted list of tuples expected
         content = sorted(content.items())
@@ -444,70 +464,67 @@ class ProcessIndex(Index):
 
 class SystemDomain(Domain):
 
-    name = 'system'
-    label = 'System definition'
-    roles = {
-        'ref': XRefRole()
-    }
+    name = "system"
+    label = "System definition"
+    roles = {"ref": XRefRole()}
     directives = {
-        'process': Process,
-        'object': Object,
+        "process": Process,
+        "object": Object,
         # 'startSubProcesses': StartSubProcessesDirective,
     }
-    indices = {
-        ProcessIndex,
-        ObjectIndex
-    }
-    initial_data = {
-        'processes': [],  # object list
-        'objects': [],  # object list
-        'process_consumes': {},
-        'process_produces': {},
+    indices = [ProcessIndex, ObjectIndex]
+    initial_data: dict = {
+        "processes": [],  # object list
+        "objects": [],  # object list
+        "process_consumes": {},
+        "process_produces": {},
     }
 
     def get_full_qualified_name(self, node):
-        return '{}.{}'.format('system', node.arguments[0])
+        return "{}.{}".format("system", node.arguments[0])
 
     def get_objects(self):
-        for obj in self.data['processes']:
-            yield(obj)
-        for obj in self.data['objects']:
-            yield(obj)
+        for obj in self.data["processes"]:
+            yield (obj)
+        for obj in self.data["objects"]:
+            yield (obj)
 
-    def resolve_xref(self, env, fromdocname, builder, typ, target, node,
-                     contnode):
-        match = [(docname, anchor)
-                 for name, sig, typ, docname, anchor, prio
-                 in self.get_objects() if sig == target]
+    def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        match = [
+            (docname, anchor)
+            for name, sig, typ, docname, anchor, prio in self.get_objects()
+            if sig == target
+        ]
 
         if len(match) > 0:
             todocname = match[0][0]
             targ = match[0][1]
 
-            return make_refnode(builder, fromdocname, todocname, targ,
-                                contnode, targ)
+            return make_refnode(builder, fromdocname, todocname, targ, contnode, targ)
         else:
-            print('Awww, found nothing')
+            print("Awww, found nothing")
             return None
 
     def add_process(self, signature, consumes, produces):
         """Add a new process to the domain."""
-        name = '{}.{}'.format('process', signature)
-        anchor = 'process-{}'.format(signature)
+        name = "{}.{}".format("process", signature)
+        anchor = "process-{}".format(signature)
 
         # self.data['recipe_ingredients'][name] = ingredients
         # name, dispname, type, docname, anchor, priority
-        self.data['processes'].append(
-            (name, signature, 'Process', self.env.docname, anchor, 0))
-        self.data['process_consumes'][name] = consumes
-        self.data['process_produces'][name] = produces
+        self.data["processes"].append(
+            (name, signature, "Process", self.env.docname, anchor, 0)
+        )
+        self.data["process_consumes"][name] = consumes
+        self.data["process_produces"][name] = produces
 
     def add_object(self, signature, ingredients):
         """Add a new object to the domain."""
-        name = '{}.{}'.format('object', signature)
-        anchor = 'object-{}'.format(signature)
+        name = "{}.{}".format("object", signature)
+        anchor = "object-{}".format(signature)
 
         # self.data['recipe_ingredients'][name] = ingredients
         # name, dispname, type, docname, anchor, priority
-        self.data['objects'].append(
-            (name, signature, 'Object', self.env.docname, anchor, 0))
+        self.data["objects"].append(
+            (name, signature, "Object", self.env.docname, anchor, 0)
+        )
